package com.mysql.cluster.mgmj;

/**
 * A NdbMgmException is the base checked exception class of the MGM/J API Exception hierarchy.
 * <br>A NdbMgmException is thrown if the error is generated in the cluster 
 * (and can either be recovered from or requires closing some open objects). 
 * It contains a message field, an error code field and a status field describing
 * the cause of the problem.
 * <br>If the error is generated by MGM/J (rather than internally in the cluster);
 * then, an error message is supplied (but no error code or error status from the cluster).
 */
public class NdbMgmException extends Exception
{

    protected static final long serialVersionUID=1L;
    protected final String message;

    /**
     * constructor for a NdbMgmException that uses a String to describe
     * the condition that caused the exception.
     * @param message
     */
    public NdbMgmException(String message)
	{
    	super(message);
	    this.message=message;
	}

    /**
     * @return String a stringified message describing the cause of the exception.
     */
    @Override
    public String toString() {
    	//if (errRef == null) { 
    		return message;
    	//}
        //return "ERROR: " + errRef.getCode() + " Message: " + message
        //+ " Classification: " + errRef.getClassification();
    }

    /**
     * @return String a stringified message describing the cause of the exception.
     */
    @Override
    public String getMessage() {
    	return toString();
    }

}
